
/* darkstat: a network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* Everything here is covered by the GPL (see the 'COPYING' file) */

#include "darkstat.h"
#include "host_db.h"
#include "port_db.h"
#include "proto.h"
#include "bignum.h"
#include "dns.h"
#include "www.h"
#include "graph.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h> /* for ntohs() and friends */
#include <net/if.h>
#include <arpa/inet.h>
#include <pcap.h>

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#if defined(sun) && (defined(__svr4__) || defined(__SVR4)) 
#include <sys/sockio.h>
#endif
#include <locale.h>

#define PCAP_TIMEOUT 1000 /* ms */
#define EXIT_SUCCESS 0
#define EXIT_FAIL 1
#define SAVE_TIME 60*5 /* secs inbetween DB dumps */

int64	num_packets, total_data;
dword	local_ip = 0, lan_ip = 0xFFFFFFFF, lan_mask = 0;
int	verbose = 0, webport = DEFAULT_WEBPORT, want_spy = 0,
	webip = DEFAULT_WEBIP, shutting_down = 0, linktype = 0;
char	*errdb_file = NULL, *db_file = NULL, *daylog_file = NULL,
	*fprefix = NULL;
time_t	t_start, t_already = 0, t_lastsave = 0;
pcap_t	*pcap = NULL, *spycap = NULL;
host_record *local_host_rec;


char *build_str(char *format, ...)
{
	va_list va;
	char buf[1024], *s;
	int count;

	va_start(va, format);
	count = vsnprintf(buf, 1024, format, va);

	if (count == -1) freakout("buffer overflow in build_str()");
	va_end(va);

	s = strdup(buf);
	if (!s) freakout("ran out of memory in build_str()");
	return s;
}



void freakout(const char *err)
{
	fprintf(stderr, "\nFatal error: %s\n\n", err);

	save_db(errdb_file);
	fprintf(stderr, "Dying now...\n");
	if (pcap) pcap_close(pcap);
	exit(EXIT_FAIL);
}



void print_addr(const dword ip)
{
	printf("%d.%d.%d.%d",
		(ip >> 24) & 255,
		(ip >> 16) & 255,
		(ip >> 8) & 255,
		(ip & 255) );
}



/* Packet decoding - hints from ACSys and tcpdump */
#define ETH_PKT_TYPE_IP 0x0800
inline word eth_pkt_type(const byte *pkt)
{
	return ntohs( ((word*)pkt)[6] );
}

inline byte *eth_pkt_strip(byte *pkt)
{
	return pkt+14;
}

#define IP_PROTO_ICMP 1 /* Internet Control Message Protocol */
#define IP_PROTO_TCP 6  /* Transmission Control Protocol */
#define IP_PROTO_UDP 17 /* User Datagram Protocol */
inline word ip_pkt_len(const byte *pkt)
{
	return ntohs( ((word*)pkt)[1] );
}

inline u_char ip_pkt_proto(const byte *pkt)
{
	return pkt[9];
}

inline dword ip_pkt_srcip(const byte *pkt)
{
	return ntohl( ((dword*)(pkt+12))[0] );
}

inline dword ip_pkt_destip(const byte *pkt)
{
	return ntohl( ((dword*)(pkt+16))[0] );
}

inline u_char *ip_pkt_strip(u_char *pkt)
{
	return pkt+20;
}

inline word tcp_pkt_srcport(const byte *pkt)
{
	return ntohs( ((word*)pkt)[0] );
}

inline word tcp_pkt_destport(const byte *pkt)
{
	return ntohs( ((word*)pkt)[1] );
}

inline word tcp_pkt_seq(const byte *pkt)
{
	return ntohl( ((dword *)pkt)[1] );
}

inline word tcp_pkt_ack(const byte *pkt)
{
	return ntohl( ((dword *)pkt)[2] );
}

inline u_char tcp_flags(const byte *pkt)
{
	return pkt[13] & 0x3F;
}

#define TCP_URG (1<<5)
#define TCP_ACK (1<<4)
#define TCP_PSH (1<<3)
#define TCP_RST (1<<2)
#define TCP_SYN (1<<1)
#define TCP_FIN (1<<0)



void interpret_linktype(u_char *pdata, u_char **ptr, word *pkt_type)
{
	/* thanks to Brian May for proper linktype handling */
	switch (linktype)
	{
	case DLT_EN10MB:
		*ptr = pdata;
		*pkt_type = eth_pkt_type(*ptr);
		break;

	case DLT_RAW:
		*ptr = pdata;
		*pkt_type = ETH_PKT_TYPE_IP;
		break;

	case DLT_FDDI:
		*ptr = pdata + 13; /* skip FDDI header */
		*ptr += 6; /* some other garbage */
		*ptr -= 12; /* now it matches up with ether */
		*pkt_type = eth_pkt_type(*ptr);
		break;

	case DLT_LINUX_SLL:
		*ptr = pdata + 14; /* skip header */
		*ptr -= 12; /* now it matches up with ether */
		*pkt_type = eth_pkt_type(*ptr);
		break;

	default:
		freakout("Unknown link type!");
	}
}



/* disassemble a packet and account for it */
void handle_pkt(u_char *user, const struct pcap_pkthdr *pheader,
	u_char *pdata)
{
	u_char *ptr;
	word pkt_type;
	dword pkt_len;
	u_char pkt_proto;
	dword pkt_srcip, pkt_destip;
	port_record *p;

	if (shutting_down) return; /* blarg */

	/* update total packet and data counters */
	i64add32(num_packets, 1);
	i64add32(total_data, (dword)(pheader->len));

	/* assert(strcmp(user, "abc") == 0); */

	interpret_linktype(pdata, &ptr, &pkt_type);

	if (pkt_type != ETH_PKT_TYPE_IP)
	{
		if (verbose)
		{
			putchar('(');
			switch (pkt_type)
			{
			case 0x0806: printf("ARP"); break;
			case 0x809b: printf("AppleTalk"); break;
			case 0x8035: printf("RARP"); break;
			case 0x814c: printf("SNMP"); break;
			case 0x86dd: printf("IPv6"); break;
			case 0x880b: printf("PPP"); break;
			default: printf("unknown (type %04X)",
					pkt_type);
			}
			printf(" packet -- ignored)\n");
			fflush(stdout);
		}
		return;
	}

	ptr = eth_pkt_strip(ptr);

	if (verbose) printf("(IP) ");

	pkt_proto = ip_pkt_proto(ptr);
	pkt_srcip = ip_pkt_srcip(ptr);
	pkt_destip = ip_pkt_destip(ptr);
	pkt_len = (dword)(ip_pkt_len(ptr));

	/* de-mangle Linux 2.4.x NAT */
	if ((pkt_srcip & lan_mask) == lan_ip) pkt_srcip = local_ip;
	if ((pkt_destip & lan_mask) == lan_ip) pkt_destip = local_ip;

	if (verbose)
	{
		printf(proto_name_short(pkt_proto));
		putchar(' ');
		print_addr(pkt_srcip);
		printf("->");
		print_addr(pkt_destip);
	}

	/* start messing with DB */
	pthread_mutex_lock(&db_mutex);
	
	/* accounting for protocols */
	if (pkt_srcip == local_ip)
		proto_transfer_out(pkt_proto, pkt_len);
	if (pkt_destip == local_ip)
		proto_transfer_in(pkt_proto, pkt_len);
	if (pkt_srcip != local_ip && pkt_destip != local_ip)
		proto_transfer_other(pkt_proto, pkt_len);

	/* accounting for hosts */
	host_transfer_out(pkt_srcip, pkt_len);
	host_transfer_in(pkt_destip, pkt_len);

	/* accounting for ports */
	if (pkt_proto == IP_PROTO_TCP)
	{
		u_char *tcp_ptr = ip_pkt_strip(ptr);
		word	srcport = tcp_pkt_srcport(tcp_ptr),
			destport = tcp_pkt_destport(tcp_ptr);
		u_char flags = tcp_flags(tcp_ptr);

		if (pkt_destip == local_ip)
		{
			/* treat all ftp-data as single port -- jib */
			if (srcport == 20) destport = 20;

			if ((flags & TCP_SYN) && !(flags ^ TCP_SYN))
				p = port_from_num(destport);
			else
				p = port_find(destport);

			if (p) port_update_in(p, pkt_len);
		}
	       	else if (pkt_srcip == local_ip)
		{
			p = port_find(srcport);
			if (p) port_update_out(p, pkt_len);
		}	

		if (verbose)
		{
			printf(" %d:%d (", srcport,destport);
		
			if (flags & TCP_URG) printf("URG ");
			if (flags & TCP_ACK) printf("ACK ");
			if (flags & TCP_PSH) printf("PSH ");
			if (flags & TCP_RST) printf("RST ");
			if (flags & TCP_SYN) printf("SYN ");
			if (flags & TCP_FIN) printf("FIN ");
			if (flags) putchar(8);
			printf(") ");

			printf("seq=%u", tcp_pkt_seq(tcp_ptr));
			if (flags & TCP_ACK)
				printf(", ack=%u", tcp_pkt_ack(tcp_ptr));
		}
	}

	/* stop messing with db */
	pthread_mutex_unlock(&db_mutex);

	/* update graph */
	pthread_mutex_lock(&graph_mutex);
	graph_rotate(pheader->ts.tv_sec);
	if (pkt_srcip == local_ip) graph_add_out(pkt_len); else
	if (pkt_destip == local_ip) graph_add_in(pkt_len);
	pthread_mutex_unlock(&graph_mutex);

	if (verbose)
	{
		printf(" size:%d\n", pkt_len);
		fflush(stdout);
	}
}



/* disassemble a packet and look for an HTTP request */
void spy_pkt(u_char *user, const struct pcap_pkthdr *pheader,
	u_char *pdata)
{
	u_char		*ptr;
	word		pkt_type;
	dword		pkt_len;
	u_char		pkt_proto;
	dword		pkt_srcip, pkt_destip;
	port_record	*p;
	u_char		*tcp_ptr;
	word		srcport, destport;
	u_char		flags;

	if (shutting_down) return; /* blarg */

	/* assert(strcmp(user, "abc") == 0); */
	assert(want_spy);

	interpret_linktype(pdata, &ptr, &pkt_type);
	ptr = eth_pkt_strip(ptr);

	pkt_srcip = ip_pkt_srcip(ptr);
	pkt_destip = ip_pkt_destip(ptr);

	tcp_ptr = ip_pkt_strip(ptr);
	destport = tcp_pkt_destport(tcp_ptr);

	spylog_packet(tcp_ptr + 20,
		      pdata + pheader->len - tcp_ptr - 20,
		      pkt_srcip, pkt_destip, destport);
}



void init_db(void)
{
	host_db_init();
	services_init();
	port_db_init();
	proto_db_init();
}



void free_db(void)
{
	host_db_free();
	port_db_free();
}



void save_db(const char *filename)
{
	dword uptime = (dword)( t_already + (time(NULL) - t_start) );
	FILE *fp = fopen(filename, "wb");

	if (!fp)
	{
		fprintf(stderr, "save_db(): Can't open %s for writing.\n",
			filename);
		exit(EXIT_FAIL);
	}

	fwrite(&uptime, sizeof(uptime), 1, fp);
	fwrite64(num_packets, fp);
	fwrite64(total_data, fp);

	pthread_mutex_lock(&db_mutex);
	host_db_save(fp);
	proto_db_save(fp);
	port_db_save(fp);
	pthread_mutex_unlock(&db_mutex);

	pthread_mutex_lock(&graph_mutex);
	graph_save(fp);
	pthread_mutex_unlock(&graph_mutex);

	fclose(fp);
}



int load_from_file(FILE *fp)
{
	int read;
	dword uptime;

	read = fread(&uptime, sizeof(uptime), 1, fp);
	t_already = (time_t)uptime;
	if (!read) return 0;

	fread64(num_packets, fp, read);
	if (!read) return 0;

	fread64(total_data, fp, read);
	if (!read) return 0;

	if (!host_db_load(fp)) return 0;
	if (!proto_db_load(fp)) return 0;
	if (!port_db_load(fp)) return 0;
	if (!graph_load(fp)) return 0;

	return 1;
}



void load_db(char *filename)
{
	FILE *fp = fopen(filename, "r");
	int okay = 1, read;
	
	if (!fp) {
		printf("Can't load db from %s, starting from scratch.\n",
			filename);
		return;
	}

	if (!load_from_file(fp))
	{
		fprintf(stderr, "Fatal error: corrupt db in %s.\n"
			"Please remove database and restart darkstat.\n",
			filename);
		fclose(fp);
		exit(EXIT_FAIL);
	}

	fclose(fp);
	printf("Loaded %s.\n", filename);
}



RETSIGTYPE term_signal(int signum)
{
	if (shutting_down)
	{
		printf("Caught a redundant SIGTERM. Ignored.\n");
		return;
	}
	shutting_down = 1;
	printf("Caught SIGTERM, syncing threads...\n");

	pthread_join(www_thread, NULL);
	pthread_join(dns_thread, NULL);

	printf("Synced!  Dumping DB...\n");

	save_db(db_file);
	printf("Done!\n");
	
	free(db_file);
	free(errdb_file);
	free(daylog_file);

	pthread_mutex_destroy(&db_mutex);
	pthread_mutex_destroy(&graph_mutex);

	free_db();

	/* pcap_close(pcap); */ /* causes long delay under FreeBSD */

	exit(EXIT_SUCCESS);
}



dword get_local_ip(char *inter)
{
	int tmp = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
	struct ifreq ifr;
	struct sockaddr sa;

	strcpy(ifr.ifr_name, inter);
	ifr.ifr_addr.sa_family = AF_INET;
	if (ioctl(tmp, SIOCGIFADDR, &ifr) != 0)
	{
		fprintf(stderr, "Can't get own IP address on interface %s.\n",
			inter);
		exit(101);
	}
	close(tmp);
	sa = ifr.ifr_addr;

	/* linux/socket.h:
	 * struct sockaddr {
	 *	sa_family_t     sa_family;      * address family, AF_xxx       
	 *	char            sa_data[14];    * 14 bytes of protocol address 
	 */

	/* linux/in.h:
	 * struct sockaddr_in {
	 * sa_family_t           sin_family;     * Address family
	 *	unsigned short int    sin_port;       * Port number
	 *	struct in_addr        sin_addr;       * Internet address
	 * * Pad to size of `struct sockaddr'.
	 *
	 * * Internet address.
	 * struct in_addr {
	 *	__u32   s_addr;
	 */

	return ntohl( ((struct sockaddr_in*)&sa)->sin_addr.s_addr );
}



int main(int argc, char **argv)
{
	char *netdev = NULL, *spydev, err[PCAP_ERRBUF_SIZE];
	int i;
	
	setlocale(LC_ALL, "");
	setlocale(LC_NUMERIC, "en_US");
#if ENABLE_NLS
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
	bind_textdomain_codeset(PACKAGE, "UTF-8");
#endif

	printf(PACKAGE " v" VERSION " using libpcap v%d.%d\n",
		PCAP_VERSION_MAJOR, PCAP_VERSION_MINOR);

	local_ip = 0;

	/* handle commandline arguments */
	for (i=1; i<argc; i++)
	{
		if ((strcmp(argv[i], "-h") == 0) ||
			(strcmp(argv[i], "--help") == 0))
		{
			printf(
"usage: %s [-i <if>] [-p <number>] [-b <ip>] [-d <path>]\n"
"\t[-l <ip>/<mask>] [-f <ip>] [-v] [-n] [-h] [-V] [--spy <if>]\n\n"
"  -i\tspecify which interface to listen on\n"
"  -p\tweb interface port (default: %d)\n"
"  -b\twhich local IP to bind the web interface to (default: all)\n"
"  -d\tdatabase storage directory (default: working directory)\n"
"  -l\tlocal network range for correct SNAT accounting (Linux 2.4.x)\n"
"  -f\tforce local ip to <ip> (great for multihomed servers)\n"
"  -v\tverbose mode - packet dumps and more\n"
"  -n\tdon't resolve IPs to hostnames\n"
"  -h\thelp (what you're looking at now)\n"
"  -V\tversion information\n"
"  --spy\tlog HTTP requests seen on specified interface\n\n",
				argv[0], webport);

			return EXIT_SUCCESS;
		}
		else if (strcmp(argv[i], "-v") == 0) verbose = 1;
		else if (strcmp(argv[i], "-n") == 0) want_resolve = OFF;
		else if (strcmp(argv[i], "--spy") == 0)
		{
			want_spy = 1;
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Interface not specified!\n\n");
				return EXIT_FAIL;
			}
			spydev = argv[i];
		}
		else if (strcmp(argv[i], "-p") == 0)
		{
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Port not specified!\n\n");
				return EXIT_FAIL;
			}
			webport = atoi(argv[i]);
		}
		else if (strcmp(argv[i], "-b") == 0)
		{
			int i1,i2,i3,i4;

			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Bind IP not specified!\n\n");
				return EXIT_FAIL;
			}
			sscanf(argv[i], "%d.%d.%d.%d",
				&i1, &i2, &i3, &i4);

			webip = ip_from_quad(i1,i2,i3,i4);

			printf("Binding webserver to %d.%d.%d.%d\n",
				i1,i2,i3,i4);
		}
		else if (strcmp(argv[i], "-i") == 0)
		{
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Interface not specified!\n\n");
				return EXIT_FAIL;
			}
			netdev = argv[i];
		}
		else if (strcmp(argv[i], "-d") == 0)
		{
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Directory not specified!\n\n");
				return EXIT_FAIL;
			}
			fprefix = build_str("%s%s", argv[i],
				argv[i][strlen(argv[i])-1] == '/'?"":"/");
		}
		else if ((strcmp(argv[i], "-V") == 0) ||
			(strcmp(argv[i], "--version") == 0))
		{
			printf(COPYRIGHT "\n" URL "\n\n"
"This program is free software; you can redistribute it and/or modify\n"
"it under the terms of the GNU General Public License as published by\n"
"the Free Software Foundation; either version 2 of the License, or\n"
"(at your option) any later version.\n\n"

"This program is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"GNU General Public License for more details.\n\n"

"You should have received a copy of the GNU General Public License\n"
"along with this program. If not, write to the Free Software\n"
"Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"
			);
			return EXIT_SUCCESS;
		}
		else if (strcmp(argv[i], "-l") == 0)
		{
			int i1,i2,i3,i4, m1,m2,m3,m4;
			
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("LAN space not specified!\n\n");
				return EXIT_FAIL;
			}
			sscanf(argv[i], "%d.%d.%d.%d/%d.%d.%d.%d",
				&i1, &i2, &i3, &i4,
				&m1, &m2, &m3, &m4);

			lan_ip = ip_from_quad(i1,i2,i3,i4);
			lan_mask = ip_from_quad(m1,m2,m3,m4);

			printf("Local LAN is %d.%d.%d.%d/%d.%d.%d.%d\n",
				i1,i2,i3,i4, m1,m2,m3,m4);
		}
		else if (strcmp(argv[i], "-f") == 0)
		{
			int i1,i2,i3,i4;
			
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Forced IP not specified!\n\n");
				return EXIT_FAIL;
			}
			sscanf(argv[i], "%d.%d.%d.%d",
				&i1, &i2, &i3, &i4);

			local_ip = ip_from_quad(i1,i2,i3,i4);

			printf("Forced IP to %d.%d.%d.%d\n",
				i1,i2,i3,i4);
		}
		else {
			printf("Invalid argument: %s\n", argv[i]);
			return EXIT_FAIL;
		}
	}

	if (!netdev)
	{
		netdev = pcap_lookupdev(err);
		if (!netdev)
		{
			fprintf(stderr, "Error: pcap_lookupdev(): %s\n", err);
			return EXIT_FAIL;
		}
	}

	/* database files, accounting for the specified storage directory */
	if (fprefix == NULL) fprefix="";
	db_file = build_str("%s" DB_FILE_NAME, fprefix);
	errdb_file = build_str("%s" ERRDB_FILE_NAME, fprefix);
	daylog_file = build_str("%s" DAYLOG_FILE_NAME, fprefix);

	if (!local_ip) local_ip = get_local_ip(netdev);
	printf("Sniffing on device %s, local IP is ", netdev);
	print_addr(local_ip);
	putchar('\n');

	init_db();
	init_graph();
	load_db(db_file);

	local_host_rec = host_from_ip(local_ip);

	/* the only way out */
	signal(SIGTERM, term_signal);
	signal(SIGINT, term_signal);

	/* fire up threads */
	printf("Firing up threads...\n"); fflush(stdout);

	if (pthread_mutex_init(&graph_mutex, NULL) == -1)
	{
		fprintf(stderr,"Error: could not create graph mutex.\n");
		return EXIT_FAIL;
	}
	if (pthread_mutex_init(&db_mutex, NULL) == -1)
	{
		fprintf(stderr,"Error: could not create db mutex.\n");
		return EXIT_FAIL;
	}
	if (pthread_create(&dns_thread, NULL, (void*)&dns_main, NULL) == -1)
	{
		fprintf(stderr,"Error: could not create DNS thread.\n");
		return EXIT_FAIL;
	}
	if (pthread_create(&www_thread, NULL, (void*)&www_main, NULL) == -1)
	{
		fprintf(stderr,"Error: could not create WWW thread.\n");
		return EXIT_FAIL;
	}

	pcap = pcap_open_live(netdev, 1500, 1, PCAP_TIMEOUT, err);
	if (!pcap)
	{
		fprintf(stderr, "Error: pcap_open_live(%s): %s\n"
			"Are you not running as root?\n", netdev, err);
		return EXIT_FAIL;
	}

	if (want_spy)
	{
		struct bpf_program spyprog;
		char *prog;
		dword spyip;

		spycap = pcap_open_live(spydev, 1500, 1, PCAP_TIMEOUT, err);
		if (!pcap)
		{
			fprintf(stderr, "Error: pcap_open_live(%s): %s\n",
				spydev, err);
			return EXIT_FAIL;
		}

		spyip = get_local_ip(spydev);
		prog = build_str("ip proto \\tcp and not "
			"(dst host %d.%d.%d.%d and dst port %u)",
				(spyip >> 24) & 255,
				(spyip >> 16) & 255,
				(spyip >> 8) & 255,
				 spyip       & 255,
				 webport);

		if (pcap_compile(spycap, &spyprog, prog, 1, 0) == -1)
		{
			fprintf(stderr, "Error: pcap_compile(): %s\n", err);
			return EXIT_FAIL;
		}

		if (pcap_setfilter(spycap, &spyprog) == -1)
		{
			fprintf(stderr, "Error: pcap_setfilter(): %s\n", err);
			return EXIT_FAIL;
		}

		pcap_freecode(&spyprog);
		free(prog);
	}

	t_lastsave = t_start = time(NULL);
	linktype = pcap_datalink(pcap);

	if (!pcap) {
		fprintf(stderr, "Error: pcap_open_live(): %s\n", err);
		return EXIT_FAIL;
	}

	printf("Capturing traffic...\n"
	       "Point your browser at http://localhost:%d/ to see the stats."
	       "\n\n", webport);

	while (!shutting_down)
	{
		struct pcap_stat ps;

		/* capture some packets for accounting */
		if (pcap_dispatch(pcap, -1,
			(pcap_handler)handle_pkt, "abc") == -1)
		{
			fprintf(stderr, "Error: pcap_dispatch(): %s\n",
				pcap_geterr(pcap));
			return EXIT_FAIL;
		}

		/* capture some packets for webspy */
		if (want_spy)
		if (pcap_dispatch(spycap, -1,
			(pcap_handler)spy_pkt, "def") == -1)
		{
			fprintf(stderr, "Error: pcap_dispatch(): %s\n",
				pcap_geterr(spycap));
			return EXIT_FAIL;
		}

		/* print out cap statistics */	
		if (verbose)
		{
			pcap_stats(pcap, &ps);
			printf("Packets: received %d, dropped %d\n",
				ps.ps_recv, ps.ps_drop);
		}

		/* commit db to disk */
		if (time(NULL) - t_lastsave > SAVE_TIME)
		{
			t_lastsave = time(NULL);
			save_db(db_file);
		}
	}

	printf("Main thread is out of capture loop.\n");
	while (shutting_down) usleep(1000*MSEC);

	assert(0 && "Execution should never reach this point!");
	return EXIT_FAIL;
}

